#! /usr/bin/env python3
# -*- coding: utf-8 -*-

"""output timestamps of a Poisson process"""

##############################################################################
# arguments handling #########################################################
##############################################################################


import sys
import getopt
import textwrap


DEFAULTS = {
	"rate": 1.,
	"size": 10,
}

def exit_usage(name, message=None, code=0):
	if message:
		sys.stderr.write("%s\n" % message)
	sys.stderr.write(textwrap.dedent("""\
	Usage: %(name)s [-hr:n:]
		-h  --help            print this help message then exit
		-r  --rate <rate>     rate of events (s^-1, defaults to %(rate)r)
		-n  --order  <size>   events count (2**size, size defaults to %(size)r)
	""") % dict(name=name, **DEFAULTS))
	sys.exit(code)


prog_name, *args = sys.argv
try:
	options, args = getopt.getopt(args, "hr:n:",
	                              ["help", "rate=", "order="])
except getopt.GetoptError as message:
	exit_usage(prog_name, message, 1)

if len(args) > 0:
	exit_usage(prog_name, "no argument expected", 1)

rate = DEFAULTS["rate"]
size = DEFAULTS["size"]

for opt, value in options:
	if opt in ["-h", "--help"]:
		exit_usage(prog_name)
	elif opt in ["-r", "--rate"]:
		rate = float(value)
	elif opt in ["-n", "--order"]:
		size = int(value)


##############################################################################
# data #######################################################################
##############################################################################

import numpy as np

dts = np.random.exponential(scale=1./rate, size=(2**size,))
p = np.array([-float('inf'), float('inf')], dtype=np.float64)
fp = np.memmap('poisson-%s.dat' % size, mode='w+', dtype=np.float64, shape=(len(dts)+2,))
fp[1:-1] = np.cumsum(dts)
fp[[0, -1]] = -float('inf'), float('inf')
del fp
