# -*- coding: utf-8 -*-

# imports ###################################################################

import time

import numpy as np


# histogram #################################################################

def bootstrap_bounds(ts, N, steps=1):
	"""build initial _t, _lo, and _hi arrays"""
	samples = list(int((N-1)*s/(steps-1)) for s in range(steps))
	t = ts[samples]
	lo = np.array(samples, dtype=np.int64)
	hi = np.array(samples, dtype=np.int64)
	return t, lo, hi


def update_bounds(ts, N, t, _t, _lo, _hi):
	"""update bounds by searching t in old _t"""
	if t[0] < _t[0]:
		lo_0 = _lo[0]
		step = 1
		while True:
			lo_0 -= step
			if lo_0 < 0:
				lo_0 = 0
				break
			if ts[lo_0] <= t[0]:
				break
			step *= 2
		_t[0]  = ts[lo_0]
		_lo[0] = lo_0
	if _t[-1] < t[-1]:
		hi_W = _hi[-1]
		step = 1
		while True:
			hi_W += step
			if hi_W > N-1:
				hi_W = N-1
				break
			if t[-1] <= ts[hi_W]:
				break
			step *= 2
		_t[-1]  = ts[hi_W]
		_hi[-1] = hi_W
	lo = _lo[np.searchsorted(_t, t, side='right')-1]
	hi = _hi[np.searchsorted(_t, t, side='left')]
	return t, lo, hi


def set_num_threads(n):
	raise RuntimeError('python implementation does not support multithreading')

def bisect(ts, t, lo, hi, runtime_ms):
	"""anytime bisecting t in ts using known bounds"""
	start = time.time()
	while (time.time()-start)*1000. <= runtime_ms and np.any(lo < hi):
		mid = (lo+hi)//2
		ts_mid = ts[mid]
		smaller = (t < ts_mid)
		higher  = (ts_mid <= t)
		hi[smaller] = mid[smaller]
		lo[higher] =  mid[higher]+1
	return (lo+hi)//2, hi-lo
