# -*- coding: utf-8 -*-

"""
Color schemes from ColorBrewer.

https://colorbrewer2.org
https://colorbrewer2.org/export/colorbrewer.json
"""


# imports ####################################################################

import os
import json


# utils ######################################################################

module_globals = globals()
__all__ = []

def export(name, value):
	module_globals[name] = value
	__all__.append(name)

def parse_rgb(rgb):
	assert rgb.startswith('rgb(') and rgb.endswith(')')
	rgb = rgb[len('rgb('):-len(')')]
	rgb = tuple(int(u)/255 for u in rgb.split(','))
	return rgb


# globals ####################################################################

colorbrewer = json.load(open(os.path.join(os.path.dirname(__file__), 'colorbrewer.json')))
for k in colorbrewer:
	schemes = colorbrewer[k] 
	export(k, {
		int(n): [
			parse_rgb(rgb)
			for rgb in schemes[n]
		]
		for n in schemes if n != 'type'
	})
	