import os
from OpenGL.GL import *

BS, CR, ESC, SPACE, DEL, NBSP, SHY = (chr(c) for c in [8, 13, 27, 32, 127, 160, 173])

_alphabet_v_id, _alphabet_h_id = None, None
_space = ord(SPACE)

def init():
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1)
	
	# text
	latin_1 = open(os.path.join(os.path.dirname(__file__), "latin-1.xo"))
	latin_1 = list(line.rstrip('\n') for line in latin_1)
	
	width, height = 6, 13
	global _n
	_n = int(len(latin_1)/height)
	
	def b(s):
		"""bytes for a line."""
		for k in range((len(s)-1)//8+1):
			yield sum((1 << 7-i) if b != ' ' else 0
			          for i, b in enumerate(s[k*8:(k+1)*8]))
	def bs(lines):
		"""bytes for multiple lines."""
		return [o for line in lines for o in b(line)]
	
	global _alphabet_v_id, _alphabet_h_id
	_alphabet_v_id = glGenLists(_n)
	_alphabet_h_id = glGenLists(_n)
	
	for c in range(_space, _space+_n):
		lines = latin_1[(c-_space)*height:(c-_space+1)*height]
		if c == ord(NBSP):
			lines = [" "*width] + ["o"*width]*(height-2) + [" "*width]
		elif c == ord(SHY):
			lines = [" "*width] + [" "*(width-1) + "o"]*(height-2) + [" "*width]
		bitmap = bytes(bs(reversed(lines)))
		glNewList(_alphabet_h_id+c-_space, GL_COMPILE)
		glBitmap(width, height, 0, 0, width, 0, bitmap)
		glEndList()
		bitmap = bytes(bs(zip(*lines)))
		glNewList(_alphabet_v_id+c-_space, GL_COMPILE)
		glBitmap(height, width, 0, 0, 0, width, bitmap)
		glEndList()


def display(s, X=0., Y=0., anchor="left", horizontal=True):
	anchor = {
		"left":  0,
		"center": -len(s)*3,
		"right": -len(s)*6,
	}[anchor]
	if horizontal:
		alphabet_id = _alphabet_h_id
		Xa, Ya = anchor, 0
	else:
		alphabet_id = _alphabet_v_id
		Xa, Ya = -1, anchor
	glRasterPos(X, Y)
	glBitmap(0, 0, 0, 0, Xa, Ya, None)
	glDisable(GL_MULTISAMPLE)
	for c in s.encode('latin-1', 'replace'):
		c -= _space
		if not (0 <= c < _n):
			continue
		glCallList(alphabet_id+c)
	glEnable(GL_MULTISAMPLE)
